#include "Keyhole.h"

// This sketch explores and tests everything that the Keyhole class can do.

void setup(void)
{
  Serial.begin(9600);
}

unsigned long t0, t1;

void loop(void)
{
  KEYHOLE k;
  if(k.begin(micros())) // the input argument is optional, but you need it if you're later going to use .elapsedMicros()
  {
    k.command("dummy1"); // do nothing, but let's see how long it takes to match it, up here at the top

    if(k.command("plot")) k.plotterMode = !k.plotterMode;                // toggle plotter mode on or off (this will also make variable assignment non-verbose, so you can time it under that condition too)
    if(k.command("auto")) k.autoSeconds = ( k.autoSeconds ? 0.0 : 1.0 ); // toggle once-per-second auto-report on or off

    // And now some variables. Let's start with the timing measurements:
    k.variable("t0", t0, VARIABLE_READ_ONLY); // tells you how many microseconds it cost to check for a command when there was none, the last time that happened
    k.variable("t1", t1, VARIABLE_READ_ONLY); // tells you how many microseconds the previous command took to process

    // Now with the help of an ad-hoc macro, let's create and expose one of every supported variable type
#   define TEST(TYPE, NAME) \
      static TYPE NAME; \
      k.variable(#NAME, NAME, k.plotterMode ? VARIABLE_SILENT : VARIABLE_VERBOSE);
  
    TEST(bool,       b);
    TEST(char,       c); // Note: char is considered signed by some microprocessor architectures and unsigned by others
    TEST(int8_t,    i8);
    TEST(uint8_t,   u8);
    TEST(int16_t,  i16);
    TEST(uint16_t, u16);
    TEST(int32_t,  i32);
    TEST(uint32_t, u32);
    TEST(float,      f);
    TEST(double,     d);
    TEST(String,     s);
    
    k.command("dummy2"); // do nothing, but let's see how long it takes to match it, down here at the bottom
    
    k.end();
    t1 = k.elapsedMicros(); // measure how long it took to match and carry out the command
  }
  else t0 = k.elapsedMicros(); // measure how long it took just to check the serial port and see whether a command was ready, even though it wasn't
  
}
